import 'dart:async';
import 'dart:convert';

import '../io/all.dart';

class IODriver {
  static final ZoneLocal<IODriver> zoneLocal =
      new ZoneLocal<IODriver>(defaultValue: new IODriver());

  static IODriver get current {
    return zoneLocal.current;
  }

  final PlatformInfo platformInfo;

  IODriver({this.platformInfo = const PlatformInfo()});

  bool get isWatchSupported => false;

  Stdout get stderr {
    throw new UnimplementedError();
  }

  Stdin get stdin {
    throw new UnimplementedError();
  }

  Stdout get stdout {
    throw new UnimplementedError();
  }

  Directory get systemTemp {
    throw new UnimplementedError();
  }

  Future<RawDatagramSocket> bindRawDatagramSocket(host, int port,
      {bool reuseAddress: true, bool reusePort = false, int ttl = 1}) {
    throw new UnimplementedError();
  }

  Future<RawServerSocket> bindRawServerSocket(address, int port,
      {int backlog: 0, bool v6Only = false, bool shared = false}) {
    throw new UnimplementedError();
  }

  Future<ServerSocket> bindServerSocket(address, int port,
      {int backlog: 0, bool v6Only = false, bool shared = false}) {
    throw new UnimplementedError();
  }

  Future<RawSocket> connectRawSocket(host, int port,
      {sourceAddress, Duration timeout}) {
    throw new UnimplementedError();
  }

  Future<Socket> connectSocket(host, int port,
      {sourceAddress, Duration timeout}) {
    throw new UnimplementedError();
  }

  Future<List<NetworkInterface>> listNetworkInterfaces(
      {bool includeLoopback: false,
      bool includeLinkLocal: false,
      InternetAddressType type = InternetAddressType.any}) {
    throw new UnimplementedError();
  }

  Future<List<InternetAddress>> lookupInternetAddresses(String host,
      {InternetAddressType type = InternetAddressType.any}) {
    throw new UnimplementedError();
  }

  Directory newDirectory(String path) {
    throw new UnimplementedError();
  }

  File newFile(String path) {
    throw new UnimplementedError();
  }

  FileSystemEntity newFileSystemEntity(String path) {
    throw new UnimplementedError();
  }

  InternetAddress newInternetAddress(String value) {
    throw new UnimplementedError();
  }

  Link newLink(String path) {
    throw new UnimplementedError();
  }

  Future<InternetAddress> reverseLookupInternetAddress(
      InternetAddress address) {
    throw new UnimplementedError();
  }

  Future<ProcessResult> run(String executable, List<String> arguments,
      {String workingDirectory,
      Map<String, String> environment,
      bool includeParentEnvironment = true,
      bool runInShell = false,
      Encoding stdoutEncoding = systemEncoding,
      Encoding stderrEncoding = systemEncoding}) {
    throw new UnimplementedError();
  }

  ProcessResult runSync(String executable, List<String> arguments,
      {String workingDirectory,
      Map<String, String> environment,
      bool includeParentEnvironment = true,
      bool runInShell = false,
      Encoding stdoutEncoding = systemEncoding,
      Encoding stderrEncoding = systemEncoding}) {
    throw new UnimplementedError();
  }

  Future<Process> start(String executable, List<String> arguments,
      {String workingDirectory,
      Map<String, String> environment,
      bool includeParentEnvironment = true,
      bool runInShell = false,
      ProcessStartMode mode = ProcessStartMode.normal}) {
    throw new UnimplementedError();
  }

  Future<ConnectionTask<RawSocket>> startConnectRawSocket(host, int port,
      {sourceAddress}) {
    throw new UnimplementedError();
  }

  Future<ConnectionTask<Socket>> startConnectSocket(host, int port,
      {sourceAddress}) {
    throw new UnimplementedError();
  }
}

class PlatformInfo {
  final int numberOfProcessors;
  final String pathSeparator;
  final String localeName;
  final String operatingSystem;
  final String operatingSystemVersion;
  final String localHostname;
  final Map<String, String> environment;
  final String executable;
  final String resolvedExecutable;
  final Uri script;
  final List<String> executableArguments;
  final String packageRoot;
  final String packageConfig;
  final String version;

  const PlatformInfo({
    this.numberOfProcessors = 1,
    this.pathSeparator = "/",
    this.localeName = "en",
    this.operatingSystem = "",
    this.operatingSystemVersion = "",
    this.localHostname = "",
    this.environment = const <String, String>{},
    this.executable = "",
    this.resolvedExecutable = "",
    this.script = null,
    this.executableArguments = const <String>[],
    this.packageRoot = "",
    this.packageConfig = "",
    this.version = "2.0.0",
  });
}

/// Enables zone-scoped values.
class ZoneLocal<T> {
  /// Default value.
  T defaultValue;

  bool _hasForked = false;

  /// Key used with [Zone.fork].
  final Object zoneValueKey = Object();

  ZoneLocal({this.defaultValue});

  /// Returns current value.
  ///
  /// If values of [Zone.current] or its ancestor is not a return value of
  /// [forkWithValue], [defaultValue] will be returned.
  T get current {
    if (_hasForked) {
      final value = Zone.current[zoneValueKey];
      if (value != null) {
        return value;
      }
    }
    return defaultValue;
  }

  /// Creates a new zone with [Zone.fork].
  Zone forkWithValue(T value) {
    _hasForked = true;
    return Zone.current.fork(zoneValues: {
      zoneValueKey: value,
    });
  }
}
